function [nodes_x,nodes_y,weights]=cubature_2D(basis_type,polygon_type,...
    polygon_sides,spline_order_vett,SPLtypestring,rotation,...
    P,Q,cubature_type,cumulative,a_trap,b_trap,g1,g2)

%--------------------------------------------------------------------------
% INPUT.
% See "driver_afek".
%
% OUTPUT.
% nodes_x,nodes_y: (nodes_x,nodes_y) are cubature points.
% weights: cubature weights, wrt cubature nodes.
%--------------------------------------------------------------------------

switch polygon_type

    case -1

        [pts,weights]=cubature_trapezoidal_region(100,cubature_type,...
            a_trap,b_trap,g1,g2);
        nodes_x=pts(:,1);
        nodes_y=pts(:,2);

    case 0

        [nodes_x,nodes_y,weights]=cubature_disk(100, cubature_type);

    case 18
        [nodes_x,weights]=quadrature_rules_1D(100,4);
        nodes_y=zeros(size(nodes_x,2));

    otherwise

        reference_cubature=2;

        switch reference_cubature

            case 1
                % Splinegauss with pre-chosen rotation.
                [nodes_x,nodes_y,weights]=splinegauss_2009(100,...
                    polygon_sides,rotation,P,Q,spline_order_vett,...
                    cumulative,SPLtypestring,cubature_type);

            case 2
            
                polygon_vertices=polygon_sides(1:end,:);
                [cub_pts,weights,err]=polygon_cubature(polygon_vertices,10);
                nodes_x=cub_pts(:,1);
                nodes_y=cub_pts(:,2);

                % IF ERR == 1, THE ROUTINE polygon_cubature HAS FOUND SOME
                % PROBLEMS. WE TRY TO PATCH THEM USING SPLINEGAUSS.
                if (err == 1)
                    % Splinegauss with pre-chosen rotation.
                    [nodes_x,nodes_y,weights]=splinegauss_2009(10,...
                        polygon_sides,rotation,P,Q,spline_order_vett,...
                        cumulative,SPLtypestring,cubature_type);
                end
        end
end
