

function pts_out=triangle_map(triangle_vertices,pts_in)

%--------------------------------------------------------------------------
% OBJECT.
%----------
%
% THIS FUNCTION MAPS POINTS pts_in OF THE UNIT TRIANGLE IN THE TRIANGLE
% WITH (ORDERED) VERTICES triangle_vertices, OBTAINING THE POINTS pts_out.
%
%--------------------------------------------------------------------------
% INPUTS.
%----------
%
% triangle_vertices: vertices of a triangle. 
%
% pts_in: points on the reference triangle (0,0),(1,0),(0,1).
%          
%--------------------------------------------------------------------------
% OUTPUT.
%----------
%
% pts_out: the points "pts_in" are mapped on the triangle 
%          "triangle_vertices" by a classical map.
%
%--------------------------------------------------------------------------
% EXAMPLE.
%----------
%
% >> triangle_vertices = [0 0; 3 0; 3 2];
% >> pts_in=[0.2 0; 0.3 0.6; 0.2 0.4; 0.9 0.05];
% >> pts_out=triangle_map(triangle_vertices,pts_in)
%
% pts_out =
%
%    0.6000         0
%    2.7000    1.2000
%    1.8000    0.8000
%    2.8500    0.1000
%
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%%
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: November 17, 2009
%--------------------------------------------------------------------------

Q1=triangle_vertices(1,:);
Q2=triangle_vertices(2,:);
Q3=triangle_vertices(3,:);

b=Q1;
P=[(Q2-Q1)' (Q3-Q1)'];

pts_out_pre=(P*pts_in')';
pts_out(:,1)=b(1)+pts_out_pre(:,1);
pts_out(:,2)=b(2)+pts_out_pre(:,2);

