
function [nodes_x,nodes_y,weights,N,a,b,c,d,tri,boundary_points]=wam(M,polygon_sides,spline_order_vett,SPLtypestring,rotation,P,Q,cubature_type,...
    cumulative,wam_select,funfcn1,funfcn2,a_trap,b_trap)

%--------------------------------------------------------------------------
% OBJECT:
%----------
%
% THIS ROUTINE DEFINES CERTAIN WAMS ON UNIT DISK, POLYGONS, CURVILINEAR
% SPLINE POLYGONS.
%
%---------
% INPUT.
%---------
%
% M : DEGREE OF THE VANDERMONDE BASIS.
%
% polygon_sides : THE DEFINITION OF THE DOMAIN COMES FROM DETERMINATING
%    SOME SAMPLINGS OF THE DOMAIN.
%    MORE PRECISELY  IF THE POLYGON HAS "L" SIDES, "polygon_sides"
%    IS A VARIABLE CONTAINING ITS VERTICES, ORDERED COUNTERCLOCKWISE.
%    AS LAST ROW MUST HAVE THE COMPONENTS OF THE FIRST VERTEX.
%
%    IN OTHER WORDS, THE FIRST ROW AND LAST ROW ARE EQUAL.
%
%    "polygon_sides" IS A "L+1 x 2" MATRIX.
%                  IN THIS DEMO, THE POLYGON IS DEFINED BY THE ROUTINE
%                  "define_polygon.m".
%
%
% spline_order_vett:SPLINE ORDER AND BLOCK (VECTOR).
%
%    [spline_order_vett(i,1)=2] : PIECEWISE LINEAR SPLINES.
%    [spline_order_vett(i,1)=4] : CUBIC SPLINES
%                                (DEPENDING ON "spline_type" IN INPUT).
%    [spline_order_vett(i,1)=k] : IN THE CASE k IS NOT 2 OR 4 IT
%                                 CHOOSES THE k-TH ORDER SPLINES
%                                 (FOR ADDITIONAL HELP DIGIT "help spapi"
%                                 IN MATLAB SHELL).
%
%    [spline_order_vett(:,2)]  : VECTOR OF FINAL COMPONENTS OF A BLOCK.
%
%    EXAMPLE:
%
%    "spline_order_vett=[2 31; 4 47; 8 67]" MEANS THAT FROM THE 1st
%    VERTEX TO THE 31th VERTEX WE HAVE AN ORDER 2 SPLINE (piecewise
%    linear), FROM THE 32th VERTEX TO THE 47th WE USE A 4th ORDER
%    SPLINE (i.e. A CUBIC AND PERIODIC SPLINE BY DEFAULT), FROM THE
%    48th TO THE 67th (AND FINAL!) WE USE AN 8th ORDER SPLINE.
%
%
% SPLtypestring: IF THE MATRIX (M X 2) "spline_order_vett" HAS SOME ELEMENT
%   IN THE FIRST COLUMN EQUAL TO ONE, SPLtypestring IT DECIDES THE TYPE OF
%   END CONDITIONS OF THE CUBIC SPLINE IN THE PERTINENT BLOCK.
%   IT IS A STRING. IT CAN BE:
%
%   complete'   : match endslopes (as given in VALCONDS,
%                 with default as under *default*).
%   'not-a-knot' : make spline C^3 across first and last
%                  interior break (ignoring VALCONDS if given).
%   'periodic'   : match first and second derivatives at
%                 first data point with those at last data
%                 point (ignoring VALCONDS if given).
%   'second'     : match end second derivatives (as given
%                  in VALCONDS,with default [0 0], i.e.,
%                 as in variational).
%   'variational': set end second derivatives equal to zero
%                  (ignoring VALCONDS if given).

% rotation: SUPPOSE x_min, x_max, y_min, y_max ARE THE MINIMUM AND MAXIMUM
%    VALUES IN x AND y REACHED BY THE POLYGON, I.E.
%    "R=[x_min,x_max] x [y_min,y_max]" IS THE SMALLEST RECTANGLE WITH
%    SIDES PARALLEL TO THE AXIS x AND y, CONTAINING THE POLYGON.
%
%   [rotation=0]: GENERAL CASE, BUT OBSERVE THAT THE FUNCTION MUST BE DEFINED
%                 IN THE RECTANGLE "R" DESCRIBED ABOVE.
%   [rotation=1]: GOOD FOR CONVEX POLYGONS. FOR CONVEX POLYGONS IT SUFFICES
%                 THAT THE FUNCTION IS DEFINED IN THE POLYGON.
%   [rotation=2]: THE USER CHOOSES A SPECIAL REFERENCE SEGMENT "PQ".
%                 CHECK [1] FOR FURTHER DETAILS. SEE THE VARIABLES "P", "Q"
%                 BELOW.
%
% P,Q: IF "rotation=2" THEN THE ALGORITHM CHOOSES A PREFERRED SEGMENT
%            "PQ" HAVING "P", "Q" AS EXTREMA, AS SET BY THE USER.
%            "P" AND "Q" ARE "1 x 2" ROW VECTORS. (SEE [3] FOR DETAILS).
%
% cubature_type: IT CHOOSES THE QUADRATURE RULE USED BY SPLINEGAUSS.
%
%   [cubature_type=0]: PADUA POINTS OVER SQUARE PANELS.
%   [cubature_type=1]: TENSORIAL FEJER 1.
%   [cubature_type=2]: TENSORIAL FEJER 2.
%   [cubature_type=3]: TENSORIAL CLENSHAW CURTIS (VIA WEIGHTS).
%   [cubature_type=4]: TENSORIAL GAUSS-LEGENDRE.
%   [cubature_type=5]: TENSORIAL GAUSS-LEGENDRE-LOBATTO.
%
% cumulative: IT CHOOSES THE PARAMETRIZATION OF THE BOUNDARY WHEN IS
%   DEFINED PARAMETRICALLY VIA SPLINES (SEE [3] FOR DETAILS).
%
%   [cumulative=0]: 1:N.
%   [cumulative=1]: CUMULATIVE.
%
% wam_select: IT SELECTS THE WAM.
%
%   [wam_select] -7: Interval. 1/N^2 equispaced points.
%   [wam_select] -6: Simplex. New mesh v.1.0. (Oct. 2009).
%   [wam_select] -5: Admissible mesh from tens. grid for disc (and find).
%   [wam_select] -4: Points for a trapezoidal region (tensorial pts based).
%   [wam_select] -3: Admissible mesh from tens. grid (and inpolygon).
%   [wam_select] -2: Points for simplex (2008).
%   [wam_select] -1: Points for a trapezoidal region (Padua pts based).
%   [wam_select]  0: Points for a disk (2008-2009).
%   [wam_select]  1: Padua type panels over curvilinear polygons (via
%                    splinegauss_2009).
%   [wam_select]  2: Gaussian or Clenshaw Curtis or Fejer type
%                    over curvilinear polygons (via splinegauss_2009).
%   [wam_select]  3: Minimal triangulation WAM (pts of a reference
%                    rectangle are mapped in any triangle of the
%                    triangulation).
%   [wam_select]  4: Minimal triangulation WAM (pts of a reference triangle
%                    WAM are mapped in any triangle of the triangulation).
%   [wam_select]  5: Minimal triangulation AM (pts of a reference triangle
%                    AM are mapped in any triangle of the triangulation).
%   [wam_select]  6: Barycentric triangulation WAM (pts of a reference
%                    triangle WAM are mapped in any triangle of the
%                    triangulation). Good for star shaped or covex domains.
%
% niter: NUMBER OF TIMES THAT THE DISCRETE ORTHOGONALIZATION PROCESS TAKES
%        PLACE. IF niter=0 THEN NO DISCRETE ORTHOGONALIZATION.
%
% funfcn1,funfcn2,a_trap,b_trap: IF [polygon_type=-1] THE TRAPEZOIDAL
%   REGION IS  DESCRIBED BY A SMALLER VALUED FUNCTION "funfcn1" AND A
%   BIGGER ONE "funfcn2", AND THE "x" INTERVAL [a_trap,b_trap].
%
%----------
% OUTPUT.
%----------
%
% nodes_x,nodes_y: x,y COMPONENTS OF WAM POINTS.
% weights
% N      : WAM DEGREE.
% a,b,c,d: [a,b] x [c,d] IS THE SMALLER RECTANGLE CONTAINING THE DOMAIN AND
%          THE WAM POINTS.
% tri    : IF A TRIANGULATION IS NEEDED, "tri" CONTAINS THE INDICES OF THE
%          VERTICES NEEDED. THIS IS THE CASE OF  MINIMAL TRIANGULATION OR
%          BARYCENTRIC TRIANGULATION FOR STAR SHAPED DOMAINS.
%
%--------------------------------------------------------------------------
% FUNCTIONS USED.
%------------------
%
% 1. meshgrid (MATLAB built-in).
% 2. triangular_wam_2009.
% 3. cubature_trapezoidal_region.
% 4. inpolygon (MATLAB built-in).
% 5. pdapts.
% 6. wam_trapezoid.
% 7. pts_disk.
% 8. splinegauss_2009_intp.
% 9. minimal_triangulation.
% 10. triangle_map.
% 11. triangular_am_2009.
%
% THE FUNCTIONS triangular_wam_2009, triangular_am_2009 ARE ATTACHED
% TO THIS FILE.
%
% THE FUNCTIONS cubature_trapezoidal_region, wam_trapezoid, pts_disk,
% minimal_triangulation, triangle_map, pdpts ARE NOT ATTACHED TO THIS FILE.
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
%% Copyright (C) 2007-2009 Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Authors: Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%% Date: February 28, 2010
%--------------------------------------------------------------------------

tri=[];

N=[];

weights=[];

boundary_points=[];

switch wam_select


        % WAM ON INTERVAL [-1,1].
    case -7
        a=-1; b=1;
        c=[]; d=[]; %DUMMY VARIABLES.

        nodes_x=(a:(b-a)/M^2:b)';
        nodes_y=zeros(size(nodes_x));

        N=M^2; tri=[];

    case -6

        [pts,boundary_points]=triangular_wam_2009(M); % DEFINED AT THE END OF THIS FILE.

        nodes_x=pts(:,1);
        nodes_y=pts(:,2);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % AM ON UNIT DISK.
    case -5
        a=-1; b=1;
        c=-1; d=1;

        N=2*M^2+1;
        hx=(b-a)/N; hy=(d-c)/N;
        [nodes_x,nodes_y]=meshgrid(a:hx:b,c:hy:d);
        nodes_x=nodes_x(:);
        nodes_y=nodes_y(:);

        indexes = find(nodes_x.^2 + nodes_y.^2 <= 1);

        nodes_x=nodes_x(indexes);
        nodes_y=nodes_y(indexes);

        % WAM ON TRAPEZOIDAL REGION.
    case -4

        N=2*M;

        [pts,w]=cubature_trapezoidal_region(N,cubature_type,...
            a_trap,b_trap,funfcn1,funfcn2);

        nodes_x=pts(:,1);
        nodes_y=pts(:,2);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % AM OVER POLYGONS.
    case -3
        a=min(polygon_sides(:,1));
        b=max(polygon_sides(:,1));
        c=min(polygon_sides(:,2));
        d=max(polygon_sides(:,2));

        N=2*M^2+1;
        hx=(b-a)/N; hy=(d-c)/N;
        [nodes_x,nodes_y]=meshgrid(a:hx:b,c:hy:d);
        nodes_x=nodes_x(:);
        nodes_y=nodes_y(:);
        in = inpolygon(nodes_x,nodes_x,...
            polygon_sides(:,1),polygon_sides(:,2));
        nodes_x=nodes_x(in);
        nodes_y=nodes_y(in);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % AM OVER SIMPLEX.
    case -2

        N=2*M;

        [y1, y2] = paduapts(N);

        u=[0 1];
        v=[1 0];
        w=[0 0];

        nodes_x=(1/4)*(v(1)-u(1))*(1+y1).*(1-y2)+...
            (1/2)*(w(1)-u(1))*(1+y2)+u(1);
        nodes_y=(1/4)*(v(2)-u(2))*(1+y1).*(1-y2)+...
            (1/2)*(w(2)-u(2))*(1+y2)+u(2);

        nodes_x=nodes_x';
        nodes_y=nodes_y';

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % WAM ON A TRAPEZOIDAL REGION. BASED ON PADUA POINTS.
    case -1

        N=2*M;
        g1 = fcnchk(funfcn1);
        g2 = fcnchk(funfcn2);
        [nodes_x,nodes_y]=wam_trapezoid(N,a_trap,b_trap,g1,g2);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % WAM ON UNIT DISK.
    case 0
        N=[];
        [nodes_x,nodes_y]=pts_disk(M);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % WAM BASED ON SPLINEGAUSS (AS PANEL MANAGER) AND PADUA POINTS.
    case 1
        N=M;

        [nodes_x,nodes_y,weights]=splinegauss_2009_intp(N,polygon_sides,...
            rotation,P,Q,spline_order_vett,cumulative,SPLtypestring,...
            0);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % WAM BASED ON SPLINEGAUSS (AS PANEL MANAGER) AND QUADRATURE POINTS
        % (AS GAUSS-LEGENDRE).
    case 2
        N=M;

        [nodes_x,nodes_y,weights]=splinegauss_2009_intp(N,polygon_sides,...
            rotation,P,Q,spline_order_vett,cumulative,SPLtypestring,...
            cubature_type);

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % MINIMAL TRIANGULATION WAM (PADUA PTS.).
    case 3

        pts=polygon_sides;

        x=polygon_sides(:,1);
        y=polygon_sides(:,2);

        tri=minimal_triangulation(x,y);

        x=[x; x(1)];
        y=[y; y(1)];

        N=2*M;

        Pad = pdpts(N);
        x_sq=Pad(:,1);
        y_sq=Pad(:,2);

        wam_pts=[];

        for index_tri=1:size(tri,1)

            u=pts(tri(index_tri,1),:);
            v=pts(tri(index_tri,2),:);
            w=pts(tri(index_tri,3),:);

            wam_pts_loc_x=0.25*(v(1)-u(1)).*(1+x_sq).*(1-y_sq)+...
                0.5*(w(1)-u(1)).*(1+y_sq)+u(1);
            wam_pts_loc_y=0.25*(v(2)-u(2)).*(1+x_sq).*(1-y_sq)+...
                0.5*(w(2)-u(2)).*(1+y_sq)+u(2);

            wam_pts_loc=[wam_pts_loc_x wam_pts_loc_y];
            wam_pts=[wam_pts; wam_pts_loc];

            pts_uvw=[u; v; w];

        end

        nodes_x=wam_pts(:,1);
        nodes_y=wam_pts(:,2);

        weights=[];

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % MINIMAL TRIANGULATION WAM (WAM 2009 PTS.).
    case 4

        % SIDES: LAST VERTICES MUST NOT BE REPEATED AND BE COLUMN VECTORS.
        N=M;
        x=polygon_sides(1:end,1); y=polygon_sides(1:end,2);
        tri=minimal_triangulation(x,y);

        pts_ref=triangular_wam_2009(N); % SET OF POINTS ON REF. TRIANGLE.

        nodes_x=[]; nodes_y=[];

        for index=1:size(tri,1)

            triangle_vertices=[x(tri(index,:)) y(tri(index,:))];
            pts_out=triangle_map(triangle_vertices,pts_ref);
            nodes_x=[nodes_x; pts_out(:,1)];
            nodes_y=[nodes_y; pts_out(:,2)];

        end

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % MINIMAL TRIANGULATION AM (AM 2009 PTS.).
    case 5

        % SIDES: LAST VERTICES MUST NOT BE REPEATED AND BE COLUMN VECTORS.
        N=M;
        x=polygon_sides(1:end,1); y=polygon_sides(1:end,2);
        tri=minimal_triangulation(x,y);

        pts_ref=triangular_am_2009(N,2); % SET OF POINTS ON REF. TRIANGLE.
        % (N,1): AM TRAPZ. PTS. BASED (BIG).
        % (N,2): AM EQSPACE TENS.PTS. BASED.
        nodes_x=[]; nodes_y=[];

        for index=1:size(tri,1)

            triangle_vertices=[x(tri(index,:)) y(tri(index,:))];
            pts_out=triangle_map(triangle_vertices,pts_ref);
            nodes_x=[nodes_x; pts_out(:,1)];
            nodes_y=[nodes_y; pts_out(:,2)];

        end

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

        % BARYCENTRIC TRIANGULATION WAM (BY 2009 WAM OVER TRIANGLES)
    case 6


        % SIDES: LAST VERTICES MUST NOT BE REPEATED AND BE COLUMN VECTORS.
        N=M;
        x=polygon_sides(1:end,1); y=polygon_sides(1:end,2);
        barycenter=[sum(x)/length(x); sum(y)/length(y)];

        xx=[barycenter(1); x];
        yy=[barycenter(2); y];


        tritype=1;

        myone=ones(length(x)-1,1);

        switch tritype
            case 1
                tri=[myone (2:length(x))' [(3:length(x))'; 2] ];
            case 2
                tri=[[(3:length(x))'; 2] myone  (2:length(x))'];
            case 3
                tri=[(2:length(x))' myone  [(3:length(x))'; 2]];
        end

        pts_ref=triangular_wam_2009(N); % SET OF POINTS ON REF. TRIANGLE.

        nodes_x=[]; nodes_y=[];

        for index=1:size(tri,1)

            triangle_vertices=[xx(tri(index,:)) yy(tri(index,:))];
            pts_out=triangle_map(triangle_vertices,pts_ref);
            nodes_x=[nodes_x; pts_out(:,1)];
            nodes_y=[nodes_y; pts_out(:,2)];

        end

        a=min([nodes_x; polygon_sides(:,1)]);
        b=max([nodes_x; polygon_sides(:,1)]);
        c=min([nodes_y; polygon_sides(:,2)]);
        d=max([nodes_y; polygon_sides(:,2)]);

end





%----------------------
% triangular_wam_2009.
%----------------------

function [pts,boundary_points]=triangular_wam_2009(deg)

n1=2*deg;
n2=n1;
j1=(0:1:deg-1);
j2=(0:1:deg);
[rho,theta]=meshgrid(cos(j1*pi/n1),j2*pi/n2);

% Mapping to the simplex
B=[rho(:).*cos(theta(:)) rho(:).*sin(theta(:))];
meshT=[B(:,1).^2 B(:,2).^2];

i1=find(rho == 1);
i2=find(theta == pi/2);
i3=find(theta == 0);

nodes_x=meshT(:,1);
nodes_y=meshT(:,2);

pts=[nodes_x nodes_y; 0 0];

boundary_points=union(i1,i2);
boundary_points=union(boundary_points,i3);
boundary_points=[boundary_points; size(pts,1)];


%----------------------
% triangular_am_2009.
%----------------------

function pts=triangular_am_2009(M,am_type)

N=5*M^2;

switch am_type

    % TRAPEZOIDAL TYPE.
    case 1

        g2 = inline('1-x');
        [s,t]=meshgrid(0:(1/N):1);
        g2s=feval(g2,s);
        t=g2s.*t;
        pts=[s(:) t(:)];

        %TENSORIAL MESH IN TRIANGLE.
    case 2

        [s,t]=meshgrid(0:(1/N):1);
        NN=size(s,1);

        %******************************************************************
        % One could try:
        %
        % pts=[s(:) t(:)];
        % ind=find(pts(:,2) <=1-pts(:,1));
        % pts=[s(ind) t(ind)];
        %
        % but does not work properly (it misses some points on the
        % diagonal). Trying M=2 it misses (0.8,0.2).
        %
        % Hence, we do the same safely with integers, find the right
        % indices and then use them to construct the AM.
        %
        %******************************************************************

        [s1,t1]=meshgrid(1:NN);
        s1v=s1(:);
        t1v=t1(:);
        ind=find(s1v <= NN+1-t1v);

        pts=[s(ind) t(ind)];

end

% clf; hold on; plot(s,t,'r.'); plot(pts(:,1),pts(:,2),'b.'); hold off




